@@ -540,6 +540,7 @@ d_tcsetpgrp='undef'
 d_telldir='undef'
 d_telldirproto='undef'
 d_tgamma='undef'
+d_thread_local='undef'
 d_thread_safe_nl_langinfo_l='undef'
 d_time='define'
 d_timegm='undef'
@@ -785,6 +786,7 @@ osvers='unknown'
 otherlibdirs=' '
 package='perl5'
 perl_static_inline='static'
+perl_thread_local=''
 phostname='hostname'
 pidtype=int
 privlib='/usr/local/lib/perl5/5.35'
@@ -850,6 +852,8 @@ srand48_r_proto='0'
 srandom_r_proto='0'
 src='.'
 ssizetype=long
+st_dev_sign='1'
+st_dev_size='4'
 st_ino_sign='1'
 st_ino_size='8'
 startperl='#!perl'
